/*
 * FILE: dbgmem.h
 * 
 * ABSTRACT: defines and prototypes for debug mem functions
 * 
 * $Id: dbgmem.h,v 1.3 1999/01/01 15:30:34 rks Exp $
 * 
 * CHANGE LOG:
 * ##  who  ddMmmyy  defect  description
 * --  ---  -------  ------  -------------------------------------------------
 *  1  rks  01Jan99  --      added parameter __FUNCTION__
 *  0  rks  30Dec98  --      initial
 */ 



#ifndef dbgmem_h_INCLUDED
#define dbgmem_h_INCLUDED

#ifndef SUCCESS
#   define SUCCESS              0
#endif

#define DBG_MEM_INVALID         1
#define DBG_MEM_OVERWRITE       2


#ifdef USE_DBG_MEM

/*
 * AFAIK only gcc provides the define __FUNCTION__, for all compilers which
 * dosn't support it we define it to "UNKNOWN" here.
 * I know that names with a leading underscore are reserved for the compiler
 * implementation, so this code is not strictly ANSI.
 */
#ifndef __FUNCTION__
#   define __FUNCTION__ "UNKNOWN"
#endif

#   define Malloc(s) dbg_malloc((s), __FILE__, __LINE__, __FUNCTION__)
#   define Calloc(c,s) dbg_calloc((c), (s), __FILE__, __LINE__, __FUNCTION__)
#   define Realloc(p,s) dbg_realloc((p),(s), __FILE__, __LINE__, __FUNCTION__)
#   define Free(p) dbg_free((p), __FILE__, __LINE__, __FUNCTION__)
#   
#   define DBG_PTR_CHECK(p) dbg_ptr_check(p)
#   define DBG_MCB_WALK()   dbg_mem_walk()
#   define DBG_MEM_EXIT()   dbg_mem_exit()

void *dbg_malloc(size_t size, char *filename, int line_no, char *function);
void *dbg_calloc(size_t count, size_t size, char *filename, int line_no,
        char *function);
void *dbg_realloc(void *p, size_t size, char *filename, int line_no,
        char *function);
void  dbg_free(void *p, char *filename, int line_no, char *function);

void  dbg_mem_exit(void);
int   dbg_mcb_check(void *p);
int   dbg_mcb_walk(void);
long  dbg_ptr_check(void *p);

#else
#   define Malloc(s) malloc(s)
#   define Calloc(c,s) calloc((c), (s))
#   define Realloc(p,s) realloc((p),(s))
#   define Free(p) free(p)
#   
#   define DBG_PTR_CHECK(p) SUCCESS
#   define DBG_MCB_WALK()
#   define DBG_MEM_EXIT()


#endif


#endif /* ! dbgmem_h_INCLUDED */

/*--------[ END OF FILE dbgmem.h ]-------------------------------------------*/
