/*
 * FILE: xreg.c
 * 
 * ABSTRACT: Main file
 * 
 * Contains main function and a routine used to remove the startup pixmap
 * from the form if it is resized. 
 * Also all globals are defined here.
 * 
 * $Id: xreg.c,v 1.4 1998/12/31 15:58:49 rks Exp $
 * 
 * CHANGE LOG:
 * ##  who  ddMmmyy  defect  description
 * ==  ===  =======  ======  =================================================
 *  0  rks  28Jun98  -       initial
 */ 

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <forms.h>

#include "mask.h"
#include "tools.h"
#include "version.h"

#include "dbgmem.h"


#define MAIN
#include "globals.h"

/*--------[ file globals ]---------------------------------------------------*/

char    Cols[] = "COLS=63";

/*
 * HEY GUY, WHAT ARE YOU DOING IN MY CODE??? ;-)
 * 
 * If you did some changes just add your name to the string below.
 */
char *readme = 
"Contributors:\n"
"Ralf Schandl (ralf.schandl@t-online.de)\n"
""
;

void usage(void)
{
    fprintf(stderr, "Xreg -- a X interface for searching through Heise "
            "registers.\n");
    fprintf(stderr, "Version %s\n", VERSION);
    fprintf(stderr, "\n");
    fprintf(stderr, "Application options:\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "-T <TESTFILE>          Debugging/Selftest. Use every line"
           " in TESTFILE as a\n");
    fprintf(stderr, "                       search string. Run all searches "
            "and exit. Good if\n");
    fprintf(stderr, "                       checking for memory leaks.\n");
    fprintf(stderr, "<SEARCHWORD>           Search for SEARCHWORD on "
            "startup\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "Beside this the following XForms parameter are used:\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "Options           Value type  Meaning                   "
            "        Default\n");
    fprintf(stderr, "\n");
    fprintf(stderr, "-debug level      integer     prints debug information  "
            "        0\n");
    fprintf(stderr, "-name appname     string      changes application name  "
            "        none\n");
    fprintf(stderr, "-sync             none        requests synchronous mode"
            "(debug)  false\n");
    fprintf(stderr, "-display host:dpy string      specifies remote host     "
            "        $DISPLAY\n");
    fprintf(stderr, "-visual class     string      TrueColor, PseudoColor ..."
            "        best\n");
    fprintf(stderr, "-depth depth      integer     specifies prefered visual "
            "depth.  best\n");
    fprintf(stderr, "-vid id           long        specifies prefered visual "
            "ID\n");
    fprintf(stderr, "-private          none        forces private colormap.  "
            "        false\n");
    fprintf(stderr, "-shared           none        forces shared colormap.   "
            "        false\n");
    fprintf(stderr, "-stdcmap          none        forces standard colormap. "
            "        false\n");
    fprintf(stderr, "-double           none        enables double buffering  "
            "        false\n");
    fprintf(stderr, "-bw width         integer     changes border width      "
            "        3\n");
}


int bg_post(FL_OBJECT *ob, int ev, FL_Coord mx, FL_Coord y, int key, void *xev)
{
    if(ev == 1)
    {
        if(fd_main->pixmap)
        {
            /* We need to reset the posthandler before calling
             * fl_delete_object, because this would in turn call
             * the posthandler again.
             */
            fl_set_object_posthandler(fd_main->BG, NULL);

            fl_delete_object(fd_main->pixmap);
            sleep(1);
            fl_redraw_form(fd_main->main);
            fd_main->pixmap = NULL;
        }
    }
    
    return 0;
}



int main(int argc, char *argv[], char *env[])
{

    char argv_buffer[2096];
    int  i;
    int  rc;
    int  opt;
    long int    mask;
    FL_IOPT     defaults;
    pid_t       pid;
    
    glob_env = env;
    
    /*
     * AHA!
     * Calls to putenv("COLS=63") leads to a SIGSEGV later on. Needs to be
     * called with a global var. Good to now :-).
     * Discovered with Linux, same behavior with AIX.
     * 
     * If we set the environment var COLS here the perl script reg does word
     * warp and we don't need to do it. This only works properly for the
     * default font size.
     */
    putenv(Cols);

    /* set defaults for XForms */
    defaults.buttonFontSize = FL_MEDIUM_SIZE;
    defaults.menuFontSize = FL_MEDIUM_SIZE;
    defaults.choiceFontSize = FL_MEDIUM_SIZE;
    defaults.browserFontSize = FL_MEDIUM_SIZE;
    defaults.inputFontSize = FL_MEDIUM_SIZE;
    defaults.labelFontSize = FL_MEDIUM_SIZE;
    
    mask = FL_PDMenuFontSize | FL_PDChoiceFontSize | FL_PDBrowserFontSize |
           FL_PDInputFontSize | FL_PDLabelFontSize ;
    fl_set_defaults(mask, &defaults);


    /* let's do XForms its option processing first */
    fl_initialize(&argc, argv, 0, 0, 0);
    
    /* all remaining options are ours */
    while((opt = getopt(argc, argv, "T:")) != -1)
    {
        switch(opt)
        {
          case 'T':
            TestFile = optarg;
            break;
           
          default:
            usage();
            exit(1);
        }
    }
    
    /* It's time to fork now
     * If a testfile is given, don't fork.
     */
    if(TestFile == NULL)
    {
        pid = fork();
        if (pid > 0)        /* Parent */
        {
            exit(0);
        }
    }

    /* read config file */
    rc = read_conf();
    if(rc == 2)
    {
        /* no config file -> bring up config dialog */
        do
        {
            cbConfigure(NULL, 0);
        }while(verify_conf());
    }

    fd_main = create_form_main();
    fl_set_app_mainform(fd_main->main);
    
    cbUmlaut(fd_main->UmlautChoice, 0);
    cbCase(fd_main->CaseChoice, 0);
    cbRegEx(fd_main->RegExChoice, 0);
    cbPhonetic(fd_main->PhoneticChoice,0);
    
    if(TestFile == NULL)
    {
        /* show the first form */
        fl_show_form(fd_main->main,FL_PLACE_CENTER|FL_FREE_SIZE,FL_FULLBORDER,
                "XReg");
    }
    else
    {
        char buffer[256];
        
        sprintf(buffer, "XReg selftest: Inputfile: \"%s\"", TestFile);
        fl_show_form(fd_main->main,FL_PLACE_CENTER|FL_FREE_SIZE,FL_FULLBORDER,
                buffer);
    }

    /*
     * If searchword is given on command line start Search now, 
     * else sho contributors
     */
    if(argc > 1)
    {
        argv_buffer[0] = 0;
        for(i=1; i<argc; i++)
        {
            if(strchr(argv[i], ' '))
            {
                strcat(argv_buffer, " \"");
                strcat(argv_buffer, argv[i]);
                strcat(argv_buffer, "\"");
            }
            else
            {
                strcat(argv_buffer, " ");
                strcat(argv_buffer, argv[i]);
            }
        }

        init_browser(fd_main, argv_buffer);
    }
    else
    {
        FL_OBJECT *ob = fd_main->Output;
        fl_add_browser_line(ob, readme);
    }


    fl_set_object_posthandler(fd_main->BG, bg_post);

    if(TestFile)
    {
        cbSearchString(NULL, 0);
    }
    else
    {
        fl_do_forms();
    }
    
    /* should never be reached */
    DBG_MEM_EXIT();
    return 0;
}

/*--------[ END OF FILE xreg.c ]-------------------------------------------*/
